@@ -5,25 +5,30 @@ from typing import Union
 
 import webbrowser
 
-from .data import DWCredentials, SnowflakeCredentials, SnowflakeDataWarehouse
-from .errors import ParameterException
-from .main import RasgoQL
-from .version import __version__
+from rasgoql.data import (
+    DWCredentials,
+    BigQueryCredentials, BigQueryDataWarehouse,
+    SnowflakeCredentials, SnowflakeDataWarehouse,
+    PostgresCredentials, PostgresDataWarehouse
+)
+from rasgoql.errors import ParameterException
+from rasgoql.main import RasgoQL
+from rasgoql.version import __version__
 
 __all__ = [
     '__version__',
-    'open_docs',
+    'docs',
     'connect',
 ]
 
 DW_MAP = {
     'snowflake': SnowflakeDataWarehouse,
-    #'bigquery': BigQueryDataWarehouse,
-    #'postgres': PostgresDataWarehouse,
+    'bigquery': BigQueryDataWarehouse,
+    'postgresql': PostgresDataWarehouse,
 }
 
 
-def open_docs():
+def docs():
     """
     Open Rasgo Docs page
     """
