@@ -3,30 +3,32 @@ Snowflake DataWarehouse classes
 """
 import logging
 import os
-import re
 from typing import List, Optional, Union
 
 import json
 import pandas as pd
-from snowflake import connector
-from snowflake.connector.pandas_tools import write_pandas
 
+from rasgoql.data.base import DataWarehouse, DWCredentials
 from rasgoql.errors import (
-    DWConnectionError, DWQueryError,
-    ParameterException, SQLException,
-    TableAccessError, TableConflictException
+    DWCredentialsWarning, DWConnectionError, DWQueryError,
+    PackageDependencyWarning, ParameterException,
+    SQLWarning, TableAccessError, TableConflictException
 )
+from rasgoql.imports import sf_connector, write_pandas
 from rasgoql.primitives.enums import (
-    check_response_type, check_table_type, check_write_method
+    check_response_type, check_write_method, check_write_table_type
 )
 from rasgoql.utils.creds import load_env, save_env
 from rasgoql.utils.df import cleanse_sql_dataframe, generate_dataframe_ddl
-from rasgoql.utils.sql import is_scary_sql, magic_fqtn_handler, parse_fqtn
-
-from .base import DataWarehouse, DWCredentials
+from rasgoql.utils.messaging import verbose_message
+from rasgoql.utils.sql import (
+    is_scary_sql, magic_fqtn_handler,
+    parse_fqtn, parse_namespace,
+    validate_namespace
+)
 
 logging.basicConfig()
-logger = logging.getLogger(__name__)
+logger = logging.getLogger('Snowflake DataWarehouse')
 logger.setLevel(logging.INFO)
 
 
@@ -46,6 +48,11 @@ class SnowflakeCredentials(DWCredentials):
             database: str,
             schema: str
         ):
+        if sf_connector is None:
+            raise PackageDependencyWarning(
+                'Missing a required python package to run Snowflake. '
+                'Please download the Snowflake package by running: '
+                'pip install rasgoql[snowflake]')
         self.account = account
         self.user = user
         self.password = password
@@ -60,6 +67,8 @@ class SnowflakeCredentials(DWCredentials):
                 "account": self.account,
                 "user": self.user,
                 "role": self.role,
+                "database": self.database,
+                "schema": self.schema,
                 "warehouse": self.warehouse,
             }
         )
@@ -67,20 +76,32 @@ class SnowflakeCredentials(DWCredentials):
     @classmethod
     def from_env(
             cls,
-            file_path: str = None
+            filepath: str = None
         ) -> 'SnowflakeCredentials':
         """
         Creates an instance of this Class from a .env file on your machine
         """
-        load_env(file_path)
+        load_env(filepath)
+        account = os.getenv('SNOWFLAKE_ACCOUNT')
+        user = os.getenv('SNOWFLAKE_USER')
+        password = os.getenv('SNOWFLAKE_PASSWORD')
+        role = os.getenv('SNOWFLAKE_ROLE')
+        warehouse = os.getenv('SNOWFLAKE_WAREHOUSE')
+        database = os.getenv('SNOWFLAKE_DATABASE')
+        schema = os.getenv('SNOWFLAKE_SCHEMA')
+        if not all([account, user, password, role, warehouse, database, schema]):
+            raise DWCredentialsWarning(
+                'Your env file is missing expected credentials. Consider running '
+                'SnowflakeCredentials(*args).to_env() to repair this.'
+            )
         return cls(
-            os.getenv('snowflake_account'),
-            os.getenv('snowflake_user'),
-            os.getenv('snowflake_password'),
-            os.getenv('snowflake_role'),
-            os.getenv('snowflake_warehouse'),
-            os.getenv('snowflake_database'),
-            os.getenv('snowflake_schema')
+            account,
+            user,
+            password,
+            role,
+            warehouse,
+            database,
+            schema
         )
 
     def to_dict(self) -> dict:
@@ -99,20 +120,22 @@ class SnowflakeCredentials(DWCredentials):
 
     def to_env(
             self,
-            file_path: str = None,
+            filepath: str = None,
             overwrite: bool = False
         ):
         """
         Saves credentials to a .env file on your machine
         """
-        creds = f'snowflake_account={self.account}\n'
-        creds += f'snowflake_user={self.user}\n'
-        creds += f'snowflake_password={self.password}\n'
-        creds += f'snowflake_role={self.role}\n'
-        creds += f'snowflake_warehouse={self.warehouse}\n'
-        creds += f'snowflake_database={self.database}\n'
-        creds += f'snowflake_schema={self.schema}\n'
-        return save_env(creds, file_path, overwrite)
+        creds = {
+            "SNOWFLAKE_ACCOUNT": self.account,
+            "SNOWFLAKE_USER": self.user,
+            "SNOWFLAKE_PASSWORD": self.password,
+            "SNOWFLAKE_ROLE": self.role,
+            "SNOWFLAKE_WAREHOUSE": self.warehouse,
+            "SNOWFLAKE_DATABASE": self.database,
+            "SNOWFLAKE_SCHEMA": self.schema,
+        }
+        return save_env(creds, filepath, overwrite)
 
 
 class SnowflakeDataWarehouse(DataWarehouse):
@@ -125,13 +148,38 @@ class SnowflakeDataWarehouse(DataWarehouse):
     def __init__(self):
         super().__init__()
         self.credentials: dict = None
-        self.connection: connector.SnowflakeConnection = None
+        self.connection: sf_connector.SnowflakeConnection = None
         self.default_database = None
         self.default_schema = None
 
     # ---------------------------
     # Core Data Warehouse methods
     # ---------------------------
+    def change_namespace(
+            self,
+            namespace: str
+        ) -> None:
+        """
+        Changes the default namespace of your connection
+
+        Params:
+        `namespace`: str:
+            namespace (database.schema)
+        """
+        namespace = self._validate_namespace(namespace)
+        database, schema = parse_namespace(namespace)
+        try:
+            self.execute_query(f'USE DATABASE {database}')
+            self.execute_query(f'USE SCHEMA {schema}')
+            self.default_namespace = namespace
+            self.default_database = database
+            self.default_schema = schema
+            verbose_message(
+                f"Namespace reset to {self.default_namespace}",
+                logger
+            )
+        except Exception as e:
+            self._error_handler(e)
 
     def connect(
             self,
@@ -158,13 +206,13 @@ class SnowflakeDataWarehouse(DataWarehouse):
             self.credentials = credentials
             self.default_database = credentials.get('database')
             self.default_schema = credentials.get('schema')
-            self.connection = connector.connect(**credentials)
-        except connector.errors.DatabaseError as e:
-            raise DWConnectionError(e)
-        except connector.errors.ForbiddenError as e:
-            raise DWConnectionError(e)
+            self.connection = sf_connector.connect(**credentials)
+            verbose_message(
+                "Connected to Snowflake",
+                logger
+            )
         except Exception as e:
-            raise e
+            self._error_handler(e)
 
     def close_connection(self):
         """
@@ -174,11 +222,12 @@ class SnowflakeDataWarehouse(DataWarehouse):
             if self.connection:
                 self.connection.close()
             self.connection = None
-            logger.info("Connection to Snowflake closed")
-        except connector.errors.DatabaseError as e:
-            raise DWConnectionError(e)
+            verbose_message(
+                "Connection to Snowflake closed",
+                logger
+            )
         except Exception as e:
-            raise e
+            self._error_handler(e)
 
     def create(
             self,
@@ -203,17 +252,24 @@ class SnowflakeDataWarehouse(DataWarehouse):
             and you know you want to overwrite it
             WARNING: This will completely overwrite data in the existing table
         """
-        table_type = check_table_type(table_type)
-        fqtn = magic_fqtn_handler(fqtn, self.default_database, self.default_schema)
+        table_type = check_write_table_type(table_type)
+        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
         if self._table_exists(fqtn) and not overwrite:
             msg = f'A table or view named {fqtn} already exists. ' \
                    'If you are sure you want to overwrite it, ' \
                    'pass in overwrite=True and run this function again'
             raise TableConflictException(msg)
-        query = f'CREATE OR REPLACE {table_type} {fqtn} AS {sql}'
+        query = f"CREATE OR REPLACE {table_type} {fqtn} COMMENT='rasgoql' AS {sql}"
         self.execute_query(query, acknowledge_risk=True, response='None')
         return fqtn
 
+    @property
+    def default_namespace(self) -> str:
+        """
+        Returns the default database.schema of this connection
+        """
+        return f'{self.default_database}.{self.default_schema}'
+
     def execute_query(
             self,
             sql: str,
@@ -238,9 +294,11 @@ class SnowflakeDataWarehouse(DataWarehouse):
                   'potentially dangerous or data-altering operation.' \
                   'If you are positive you want to run this, ' \
                   'pass in acknowledge_risk=True and run this function again.'
-            raise SQLException(msg)
-        logger.debug('>>>Executing SQL:')
-        logger.debug(sql)
+            raise SQLWarning(msg)
+        verbose_message(
+            f"Executing query: {sql}",
+            logger
+        )
         if response == 'DICT':
             return self._execute_dict_cursor(sql)
         if response == 'DF':
@@ -257,7 +315,7 @@ class SnowflakeDataWarehouse(DataWarehouse):
         `fqtn`: str:
             Fully-qualified Table Name (database.schema.table)
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_database, self.default_schema)
+        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
         sql = f"SELECT GET_DDL('TABLE', '{fqtn}') AS DDL"
         query_response = self.execute_query(sql, response='dict')
         return query_response[0]['DDL']
@@ -278,7 +336,7 @@ class SnowflakeDataWarehouse(DataWarehouse):
             is rasgo object: bool
             object type: [table|view|unknown]
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_database, self.default_schema)
+        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
         database, schema, table = parse_fqtn(fqtn)
         sql = f"SHOW OBJECTS LIKE '{table}' IN {database}.{schema}"
         result = self.execute_query(sql, response='dict')
@@ -292,7 +350,8 @@ class SnowflakeDataWarehouse(DataWarehouse):
 
     def get_schema(
             self,
-            fqtn: str
+            fqtn: str,
+            create_sql: str = None
         ) -> dict:
         """
         Return the schema of a table or view
@@ -300,11 +359,28 @@ class SnowflakeDataWarehouse(DataWarehouse):
         Params:
         `fqtn`: str:
             Fully-qualified table name (database.schema.table)
+        `create_sql`: str:
+            A SQL select statement that will create the view. If this param is passed
+            and the fqtn does not already exist, it will be created and profiled based
+            on this statement. The view will be dropped after profiling
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_database, self.default_schema)
-        sql = f"DESC TABLE {fqtn}"
-        query_response = self.execute_query(sql, response='dict')
-        return query_response
+        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
+        desc_sql = f"DESC TABLE {fqtn}"
+        response = []
+        try:
+            if self._table_exists(fqtn):
+                query_response = self.execute_query(desc_sql, response='dict')
+            elif create_sql:
+                self.create(create_sql, fqtn, table_type='view')
+                query_response = self.execute_query(desc_sql, response='dict')
+                self.execute_query(f'DROP VIEW {fqtn}', response='none', acknowledge_risk=True)
+            else:
+                raise TableAccessError(f'Table {fqtn} does not exist or cannot be accessed.')
+            for row in query_response:
+                response.append((row['name'], row['type']))
+            return response
+        except Exception as e:
+            self._error_handler(e)
 
     def list_tables(
             self,
@@ -345,14 +421,18 @@ class SnowflakeDataWarehouse(DataWarehouse):
         `limit`: int:
             Records to return
         """
-        return self.execute_query(f'{sql} LIMIT {limit}', response='df', acknowledge_risk=True)
+        return self.execute_query(
+            f'{sql} LIMIT {limit}',
+            response='df',
+            acknowledge_risk=True
+        )
 
     def save_df(
             self,
             df: pd.DataFrame,
             fqtn: str,
             method: str = None
-        ):
+        ) -> str:
         """
         Creates a table in Snowflake from a pandas Dataframe
 
@@ -371,7 +451,7 @@ class SnowflakeDataWarehouse(DataWarehouse):
         """
         if method:
             method = check_write_method(method)
-        fqtn = magic_fqtn_handler(fqtn, self.default_database, self.default_schema)
+        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
         table_exists = self._table_exists(fqtn)
         if table_exists and not method:
             msg = f"A table named {fqtn} already exists. " \
@@ -384,16 +464,17 @@ class SnowflakeDataWarehouse(DataWarehouse):
             # Issue a create or replace statement before we insert data
             if not table_exists or method == 'REPLACE':
                 create_stmt = generate_dataframe_ddl(df, fqtn)
+                create_stmt += " COMMENT='rasgoql' "
                 self.execute_query(create_stmt, response='None', acknowledge_risk=True)
-            success, chunks, rows, output = write_pandas(
+            _success, _chunks, _rows, _output = write_pandas(
                 conn=self.connection,
                 df=df,
                 table_name=fqtn,
                 quote_identifiers=False
             )
-            return success, chunks, rows, output
+            return fqtn
         except Exception as e:
-            raise e
+            self._error_handler(e)
 
     # ---------------------------
     # Core Data Warehouse helpers
@@ -409,51 +490,67 @@ class SnowflakeDataWarehouse(DataWarehouse):
         `fqtn`: str:
             Fully-qualified table name (database.schema.table)
         """
-        fqtn = magic_fqtn_handler(fqtn, self.default_database, self.default_schema)
+        fqtn = magic_fqtn_handler(fqtn, self.default_namespace)
         do_i_exist, _, _ = self.get_object_details(fqtn)
         return do_i_exist
 
     def _validate_namespace(
             self,
             namespace: str
-        ):
+        ) -> str:
         """
         Checks a namespace string for compliance with Snowflake format
 
         Params:
         `namespace`: str:
-            namespace (database.schema.table)
+            namespace (database.schema)
         """
-        # Does this match a 'string.string' pattern?
-        if re.match(r'\w+\.\w+', namespace):
-            return
-        raise ParameterException("Snowflake namespaces should be format: DATABASE.SCHEMA")
+        try:
+            validate_namespace(namespace)
+            return namespace.upper()
+        except ValueError:
+            raise ParameterException("Snowflake namespaces should be format: DATABASE.SCHEMA")
 
     # --------------------------
     # Snowflake specific helpers
     # --------------------------
-    def _execute_string(
+    def _error_handler(
             self,
-            query: str, *,
-            ignore_results: bool = False
-        ) -> List[tuple]:
+            exception: Exception,
+            query: str = None
+        ) -> None:
         """
-        Execute a query string against the Data Warehouse connection and fetch all results
+        Handle Snowflake exceptions that need additional info
         """
-        query_returns = []
-        cursor = None
-        try:
-            for cursor in self.connection.execute_string(query, return_cursors=(not ignore_results)):
-                for query_return in cursor:
-                    query_returns.append(query_return)
-            return query_returns
-        except connector.errors.ProgrammingError as e:
-            raise DWQueryError(e)
-        except Exception as e:
-            raise e
-        finally:
-            if cursor:
-                cursor.close()
+        verbose_message(
+            f"Exception occurred while running query: {query}",
+            logger
+        )
+        if exception is None:
+            return
+        if isinstance(exception, sf_connector.errors.ProgrammingError):
+            if exception.errno == 3001:
+                raise TableAccessError(
+                    'You do not have access to operate on this object. '
+                    'Two possible ways to resolve: '
+                    'Connect with different credentials that have the proper access. '
+                    'Or run `.change_namespace` on your SQLChain to write it to a '
+                    'namespace your credentials can access'
+                ) from exception
+        if isinstance(exception, sf_connector.errors.DatabaseError):
+            if exception.errno == 250001:
+                raise DWConnectionError(
+                    'Invalid username / password, please check that your '
+                    'credentials are correct and try to reconnect.'
+                ) from exception
+        if isinstance(exception, sf_connector.errors.ServiceUnavailableError):
+            raise DWConnectionError(
+                'Snowflake is unavailable. Please check that your are using '
+                'a valid account identifier, that you have internet access, and '
+                'that http connections to Snowflake are whitelisted in your env. '
+                'Finally check https://status.snowflake.com/ for outage status.'
+            ) from exception
+        raise exception
 
     def _execute_dict_cursor(
             self,
@@ -472,13 +569,11 @@ class SnowflakeDataWarehouse(DataWarehouse):
         """
         cursor = None
         try:
-            cursor = self.connection.cursor(connector.DictCursor)
+            cursor = self.connection.cursor(sf_connector.DictCursor)
             query_return = cursor.execute(query).fetchall()
             return query_return
-        except connector.errors.ProgrammingError as e:
-            raise DWQueryError(e)
         except Exception as e:
-            raise e
+            self._error_handler(e)
         finally:
             if cursor:
                 cursor.close()
@@ -496,10 +591,29 @@ class SnowflakeDataWarehouse(DataWarehouse):
             cursor = self.connection.cursor()
             query_return = cursor.execute(query, params).fetch_pandas_all()
             return query_return
-        except connector.errors.ProgrammingError as e:
-            raise DWQueryError(e)
         except Exception as e:
-            raise e
+            self._error_handler(e)
+        finally:
+            if cursor:
+                cursor.close()
+
+    def _execute_string(
+            self,
+            query: str,
+            ignore_results: bool = False
+        ) -> List[tuple]:
+        """
+        Execute a query string against the Data Warehouse connection and fetch all results
+        """
+        query_returns = []
+        cursor = None
+        try:
+            for cursor in self.connection.execute_string(query, return_cursors=(not ignore_results)):
+                for query_return in cursor:
+                    query_returns.append(query_return)
+            return query_returns
+        except Exception as e:
+            self._error_handler(e)
         finally:
             if cursor:
                 cursor.close()
