@@ -3,21 +3,18 @@ Transform rendering methods
 """
 import functools
 import inspect
-import logging
-import re
 from itertools import combinations, permutations, product
-from typing import Callable, Dict, List, Optional
+import re
+import os
+from typing import Callable, Dict, List, Tuple, Optional
 
 import jinja2
 import pandas as pd
 import rasgotransforms as rtx
 
 from rasgoql.errors import TransformRenderingError
-from .enums import check_table_type
-
-logging.basicConfig()
-logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
+from rasgoql.primitives.enums import check_write_table_type
+from rasgoql.utils.dbt import save_model_file
 
 RUN_QUERY_LIMIT = 100
 JINJA_ENV = jinja2.Environment(extensions=['jinja2.ext.do', 'jinja2.ext.loopcontrols'])
@@ -91,6 +88,28 @@ def assemble_view_chain(
         view_list.append(t_view_str)
     return '\n'.join(view_list)
 
+def create_dbt_files(
+        transforms: List['Transform'],
+        schema: List[Tuple[str, str]],
+        output_directory: str = None,
+        file_name: str = None,
+        config_args: dict = None,
+        include_schema: bool = False
+) -> str:
+    """
+    Saves a dbt_project.yml and model.sql files to a directory
+    """
+    output_directory = output_directory or os.getcwd()
+    file_name = file_name or f'{transforms[-1].output_alias}.sql'
+    return save_model_file(
+        sql_definition=assemble_cte_chain(transforms),
+        output_directory=output_directory,
+        file_name=file_name,
+        config_args=config_args,
+        include_schema=include_schema,
+        schema=schema
+    )
+
 def generate_transform_sql(
         name: str,
         arguments: dict,
@@ -101,7 +120,7 @@ def generate_transform_sql(
     """
     Returns the SQL for a Transform with applied arguments
     """
-    templates = rtx.serve_rasgo_transform_templates()
+    templates = rtx.serve_rasgo_transform_templates(dw.dw_type)
     udt: 'TransformTemplate' = [t for t in templates if t.name == name][0]
     if not udt:
         raise TransformRenderingError(f'Cannot find a transform named {name}')
@@ -114,7 +133,7 @@ def generate_transform_sql(
                                                'the "sql" argument with template to apply')
         return render_template(name, source_code, arguments, source_table, running_sql, dw)
     except Exception as e:
-        raise TransformRenderingError(e)
+        raise TransformRenderingError(e) from e
 
 def render_template(
         name: str,
@@ -147,6 +166,7 @@ def _cleanse_template_symbol(
     delete anything that is not letters, numbers, or underscores
     if first character is a number, add an underscore to the beginning
     """
+    symbol = str(symbol)
     symbol = symbol.strip()
     symbol = symbol.replace(' ', '_').replace('-', '_')
     symbol = symbol.upper()
@@ -244,7 +264,7 @@ def _run_query(
             dw.execute_query(create_sql, response='none', acknowledge_risk=True)
         return dw.execute_query(query, response='df', acknowledge_risk=True)
     except Exception as e:
-        raise TransformRenderingError(e)
+        raise
     finally:
         if running_sql:
             drop_sql = f"DROP VIEW IF EXISTS {source_table}"
@@ -258,8 +278,7 @@ def _set_create_statement(
     Returns a create statement or a blank string
     """
     if table_type:
-        table_type = check_table_type(table_type)
-    if table_type in ['TABLE', 'VIEW']:
+        table_type = check_write_table_type(table_type)
         return f'CREATE OR REPLACE {table_type} {fqtn} AS \n'
     return ''
 
