@@ -8,21 +8,28 @@ import pandas as pd
 import rasgotransforms as rtx
 
 from rasgoql.errors import ParameterException
-from rasgoql.utils.decorators import require_dw, require_transforms
-from rasgoql.utils.sql import parse_fqtn, random_table_name, validate_fqtn
-
-from .enums import (
-    check_render_method, check_table_type,
+from rasgoql.utils.decorators import (
+    beta, require_dw,
+    require_materialized, require_transforms
+)
+from rasgoql.utils.sql import (
+    parse_fqtn, make_namespace_from_fqtn,
+    random_table_name, validate_fqtn
+)
+from rasgoql.primitives.enums import (
+    check_render_method, check_table_type, check_write_table_type,
     TableType, TableState
 )
-from .rendering import (
-    assemble_cte_chain, assemble_view_chain,
+from rasgoql.primitives.rendering import (
+    assemble_cte_chain, assemble_view_chain, create_dbt_files,
     _gen_udt_func_docstring, _gen_udt_func_signature
 )
 
 logging.basicConfig()
-logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
+ds_logger = logging.getLogger('Dataset')
+ds_logger.setLevel(logging.INFO)
+chn_logger = logging.getLogger('SQLChain')
+chn_logger.setLevel(logging.INFO)
 
 
 class TransformableClass:
@@ -55,14 +62,14 @@ class TransformableClass:
         """
         Gather available transforms and create aliased functions for each
         """
-        for transform in rtx.serve_rasgo_transform_templates():
+        for transform in rtx.serve_rasgo_transform_templates(self._dw.dw_type):
             f = self._create_aliased_function(transform)
             setattr(self, transform.name, f)
 
     @require_dw
     def transform(
             self,
-            name: str = None,
+            name: str,
             arguments: dict = None,
             output_alias: str = None,
             **kwargs
@@ -76,10 +83,12 @@ class TransformableClass:
         if isinstance(self, Dataset):
             entry_table = self
             source_table = self.fqtn
+            namespace = self._dw.default_namespace
         # If we're transforming a Chain
         if isinstance(self, SQLChain):
             entry_table = self.entry_table
             source_table = self.output_table.fqtn
+            namespace = self.namespace
             # If we're transforming a Chain with existing transforms
             if self.transforms:
                 source_table = self.output_table.table_name
@@ -90,12 +99,12 @@ class TransformableClass:
         new_transform = Transform(
             name,
             arguments,
-            self.namespace,
+            namespace,
             output_alias,
             self._dw
         )
         transforms.append(new_transform)
-        return SQLChain(entry_table, transforms, self._dw)
+        return SQLChain(entry_table, namespace, transforms, self._dw)
 
 
 class Dataset(TransformableClass):
@@ -108,21 +117,22 @@ class Dataset(TransformableClass):
             dw: 'DataWarehouse' = None
         ):
         super().__init__(dw)
-        if not validate_fqtn(fqtn):
+        try:
+            self.fqtn: str = validate_fqtn(fqtn)
+            self.table_name: str = parse_fqtn(fqtn)[2]
+            self.namespace: str = make_namespace_from_fqtn(fqtn)
+            self._dw._validate_namespace(self.namespace)
+        except ValueError:
             raise ParameterException("Must pass in a valid 'fqtn' parameter to create a Dataset")
-        self.fqtn = fqtn
-        database, schema, table_name = parse_fqtn(fqtn)
-        self.namespace = f'{database}.{schema}'
-        self.table_name = table_name
-        self._dw._validate_namespace(self.namespace)
-
-        self.table_type = TableType.UNKNOWN
-        self.table_state = TableState.UNKNOWN
-        self.is_rasgo = False
+        self.table_type: str = TableType.UNKNOWN.value
+        self.table_state: str = TableState.UNKNOWN.value
+        self.is_rasgo: bool = False
         self._dw_sync()
 
     def __repr__(self) -> str:
-        return f"Dataset(fqtn={self.fqtn}, type={self.table_type.value})"
+        return f"Dataset(fqtn={self.fqtn}, " \
+               f"type={self.table_type}, " \
+               f"state={self.table_state})"
 
     @require_dw
     def _dw_sync(self):
@@ -131,9 +141,11 @@ class Dataset(TransformableClass):
         """
         obj_exists, is_rasgo_obj, obj_type = self._dw.get_object_details(self.fqtn)
         if obj_exists:
-            self.table_state = TableState.IN_DW
-            self.table_type = TableType[obj_type]
+            self.table_state = TableState.IN_DW.value
+            self.table_type = check_table_type(obj_type)
             self.is_rasgo = is_rasgo_obj
+        else:
+            self.table_state = TableState.IN_MEMORY.value
 
     @require_dw
     def get_schema(self) -> dict:
@@ -143,6 +155,7 @@ class Dataset(TransformableClass):
         return self._dw.get_schema(self.fqtn)
 
     @require_dw
+    @require_materialized
     def preview(self) -> pd.DataFrame:
         """
         Return a pandas DataFrame of top 10 rows
@@ -157,6 +170,7 @@ class Dataset(TransformableClass):
         return self._dw.get_ddl(self.fqtn)
 
     @require_dw
+    @require_materialized
     def to_df(self) -> pd.DataFrame:
         """
         Return a pandas DataFrame of the entire table
@@ -226,7 +240,7 @@ class Transform:
         """
         Returns the fully qualified table name the transform would create if saved
         """
-        return self.namespace + '.' + self.output_alias
+        return f'{self.namespace}.{self.output_alias}'
 
 
 class SQLChain(TransformableClass):
@@ -237,13 +251,14 @@ class SQLChain(TransformableClass):
     def __init__(
             self,
             entry_table: Dataset,
+            namespace: str,
             transforms: List[Transform] = None,
             dw: 'DataWarehouse' = None
         ) -> None:
         super().__init__(dw)
         self.entry_table = entry_table
         self.transforms = transforms
-        self.namespace = entry_table.namespace
+        self.namespace = namespace
 
     def __repr__(self) -> str:
         transform_count = len(self.transforms) if self.transforms else 0
@@ -256,19 +271,30 @@ class SQLChain(TransformableClass):
         """
         Re-sets the class's namespace
         """
-        self._dw._validate_namespace(new_namespace)
+        self._dw.change_namespace(new_namespace)
         self.namespace = new_namespace
-        logger.info(f"Namespace reset to {self.namespace}")
+        for t in self.transforms:
+            t.namespace = new_namespace
 
     @property
     def fqtn(self) -> str:
         """
-        Returns the fully qualified table name the transform would create if saved
-        
+        Returns the fully qualified table name this SQLChain would create
+        if saved in current state
+
         NOTE: This property will be dynamic until the Chain is finally saved
         """
         return self.output_table.fqtn
 
+    @require_dw
+    def get_schema(self) -> dict:
+        """
+        Return the table schema this SQLChain would create if saved in current state
+
+        NOTE: This property will be dynamic until the Chain is finally saved
+        """
+        return self._dw.get_schema(self.fqtn, self.sql())
+
     @property
     def output_table(self) -> Dataset:
         """
@@ -277,7 +303,7 @@ class SQLChain(TransformableClass):
         NOTE: This property will be dynamic until the Chain is finally saved
         """
         if self.ternimal_transform:
-            return Dataset(f"{self.ternimal_transform.fqtn}", self._dw)
+            return Dataset(self.ternimal_transform.fqtn, self._dw)
         return self.entry_table
 
     @require_dw
@@ -319,9 +345,7 @@ class SQLChain(TransformableClass):
         """
         Materializes this Transform Chain into SQL objects
         """
-        table_type = check_table_type(table_type)
-        if table_type == 'UNKNOWN':
-            raise ValueError("table_type must be 'VIEW' or 'TABLE'")
+        table_type = check_write_table_type(table_type)
         table_name = table_name or self.output_table.fqtn
         new_table = self._dw.create(
             self.sql(),
@@ -340,6 +364,50 @@ class SQLChain(TransformableClass):
             return self.transforms[-1]
         return None
 
+    @beta
+    def to_dbt(
+            self,
+            output_directory: str = None,
+            file_name: str = None,
+            config_args: dict = None,
+            include_schema: bool = False
+        ) -> str:
+        """
+        Saves a new model.sql file to your dbt models directory
+
+        Params:
+        `output_directory`: str:
+            directory to save model file
+            defaults to current working dir
+        `file_name`: str:
+            defaults to {output_alias}.sql of SQLChain
+        `include_schema`: bool:
+            Include a schema.yml file
+        `config_args`: dict:
+            key value pair of
+            dbt [config values](https://docs.getdbt.com/reference/model-configs)
+        """
+        try:
+            schema = self.get_schema()
+        except:
+            if include_schema:
+                chn_logger.warning(
+                    'Unexpected error generating the schema of this SQLChain. '
+                    'Your model.sql file will be generated without a schema.yml file. '
+                    'This is most likely a syntax issue in your SQLChain or existing view. '
+                    'Consider running your_chn.sql() to check the syntax and/or '
+                    'your_chn.save() to update the view definition in your Data Warehouse.'
+                )
+            schema = []
+        return create_dbt_files(
+            self.transforms,
+            schema,
+            output_directory,
+            file_name,
+            config_args,
+            include_schema
+        )
+
     def to_df(self) -> pd.DataFrame:
         """
         Returns data into a pandas DataFrame
